% redWaveNnInversion_GFB_m.m - This file is part of the redWaveToolbox.
% This software aims at performing redundant wavelet transformations.
% Copyright 2014 CEA
% Contributor : Jeremy Rapin (jeremy.rapin@cea.fr)
% Created on 14/7/2014, last modified on 16/7/2014
% 
% This software is governed by the CeCILL license under French law and
% abiding by the rules of distribution of free software. You can use,
% modify and/ or redistribute the software under the terms of the CeCILL
% license as circulated by CEA, CNRS and INRIA at the following URL
% "http://www.cecill.info".
% 
% As a counterpart to the access to the source code and rights to copy,
% modify and redistribute granted by the license, users are provided only
% with a limited warranty and the software's author,  the holder of the
% economic rights,  and the successive licensors have only limited
% liability.
% 
% In this respect, the user's attention is drawn to the risks associated
% with loading,  using,  modifying and/or developing or reproducing the
% software by the user in light of its specific status of free software,
% that may mean that it is complicated to manipulate,  and that also
% therefore means that it is reserved for developers and experienced
% professionals having in-depth computer knowledge. Users are therefore
% encouraged to load and test the software's suitability as regards their
% requirements in conditions enabling the security of their systems and/or
% data to be ensured and,  more generally, to use and operate it in the
% same conditions as regards security.
% 
% The fact that you are presently reading this means that you have had
% knowledge of the CeCILL license and that you accept its terms.



% S = redWaveNnInversion_GFB_m(S0, AtA, AtY, lambda, parameters)
% Solves argmin_{S>=0} ||Y-A * S||_2^2 + ||lambda .* (S W^T)||_1
% Inputs:
% using the Generalized Farward-Backward algorithm
% (requires the nGMCAlab toolbox)
%
% Inputs:
% - S0: starting point for S
% - AtA: matrix A' * A
% - AtY: matrix A' * Y
% - lambda: sparsity parameters
% Optional fields for argument "parameter":
%   - verbose : verbose mode (default: 0).
%   - MaximumIteration: maximum number of iteration of the algorithm
%   (default: 300).
%   - MaximumSubIteration: maximum number of iteration for the computation
%   of the proximal operator of ||lambda .* (S W^T)||_1 (default: 20).
%   - qmf: wavelet coefficients (use MakeONFilter from the Wavelab toolbox,
%   default: Symmlet, 4).
%   - RelativeDifferenceTolerance: relative difference tolerance for
%   convergence of the Generalized Forward-Backward iterations (default: 10^-8)
%   - L: number of scales of the wavelets (default: 3).
%   - isometric: use isometric wavelet transform (default: 0).
%   - sourcesShape: when dealing with 2D images, shape of the images (ex:
%   [128, 128])
%   - display: display function for the current iterate.

function S = redWaveNnInversion_GFB_m(S0, AtA, AtY, lambda, parameters)

if ~exist('ApplyAlgorithm', 'file')
    error('redWaveNnInversion_GFB_m requires the nGMCAlab toolbox');
end


if nargin < 4
    err('''lambda'' is required (wavelet data size).\n');
end
if nargin<5
    parameters = [];
end

if ~isfield(parameters, 'verbose')
    parameters.verbose = 0;
end
if ~isfield(parameters, 'MaximumIteration')
    parameters.MaximumIteration=300;
end
if ~isfield(parameters, 'MaximumSubIteration')
    parameters.MaximumSubIteration = 20;
end

if ~isfield(parameters, 'RelativeDifferenceTolerance')
    parameters.RelativeDifferenceTolerance = 0.00000001;
end

if ~isfield(parameters, 'qmf')
    parameters.qmf = [-0.0758, -0.0296, 0.4976, 0.8037, 0.2979, -0.0992, -0.0126, 0.0322];
    if parameters.verbose
        fprintf(1, 'Filter ''qmf'' set to default value.\n');
    end
end
if ~isfield(parameters, 'L')
    parameters.L = 3;
    if parameters.verbose
        fprintf(1, 'Number of scales ''L'' set to 3.\n');
    end
end
if ~isfield(parameters, 'isometric')
    parameters.isometric = 0;
    if parameters.verbose
        fprintf(1, 'Non-isometric mode.\n');
    end
end

options.isometric = parameters.isometric;
options.qmf = parameters.qmf;
options.L = parameters.L;
options.MaximumIteration = parameters.MaximumSubIteration;

if ~isfield(parameters, 'sourcesShape')
    options.dimensions = 2;
    s2l = @(S) S;
    l2s = @(S) S;
else
    parameters.dimensions = find([1, parameters.sourcesShape] > 1);
    s2l = @(S) reshape(S, [size(S, 1), numel(S) / size(S, 1)]);
    l2s = @(S) reshape(S, [size(S, 1), parameters.sourcesShape]);

end



innerParam.gradient = @(x) AtA * x - AtY;
innerParam.proximals{1} = @(x, threshold) s2l(redWaveThresholding(l2s(x), threshold, options));
innerParam.lambdas{1} = lambda;
innerParam.proximals{2} = @(x, lambda) max(0, x);
innerParam.lambdas{2} = 0;

innerParam.initialization.x = S0;
innerParam.RelativeDifferenceTolerance = parameters.RelativeDifferenceTolerance;
innerParam.MaximumIteration = parameters.MaximumIteration;
innerParam.LipschitzConstant = max(eig(AtA));

if isfield(parameters, 'display')
    innerParam.display = @(data) parameters.display(data.x);
end

res = ApplyAlgorithm(GeneralizedForwardBackward_alg, innerParam);
S = res.x;



end


