% redWaveThresholding_m.m - This file is part of the redWaveToolbox.
% This software aims at performing redundant wavelet transformations.
% Copyright 2014 CEA
% Contributor : Jeremy Rapin (jeremy.rapin@cea.fr)
% Created on 14/7/2014, last modified on 16/7/2014
% 
% This software is governed by the CeCILL license under French law and
% abiding by the rules of distribution of free software. You can use,
% modify and/ or redistribute the software under the terms of the CeCILL
% license as circulated by CEA, CNRS and INRIA at the following URL
% "http://www.cecill.info".
% 
% As a counterpart to the access to the source code and rights to copy,
% modify and redistribute granted by the license, users are provided only
% with a limited warranty and the software's author,  the holder of the
% economic rights,  and the successive licensors have only limited
% liability.
% 
% In this respect, the user's attention is drawn to the risks associated
% with loading,  using,  modifying and/or developing or reproducing the
% software by the user in light of its specific status of free software,
% that may mean that it is complicated to manipulate,  and that also
% therefore means that it is reserved for developers and experienced
% professionals having in-depth computer knowledge. Users are therefore
% encouraged to load and test the software's suitability as regards their
% requirements in conditions enabling the security of their systems and/or
% data to be ensured and,  more generally, to use and operate it in the
% same conditions as regards security.
% 
% The fact that you are presently reading this means that you have had
% knowledge of the CeCILL license and that you accept its terms.

% res = redWaveThresholding_m(x, lambda, options)
% computes the proximal operator of ||lambda .* (S W^T)||_1
% using the Forward-Backward algorithm
%
% Inputs:
% - x: initialization
% - lambda: sparsity parameters
% - option: option structure with optional fields:
%   - MaximumIteration: maximum number of iteration of the algorithm
%   (default: 24).
%   - qmf: wavelet coefficients (use MakeONFilter from the Wavelab toolbox,
%   default: Symmlet, 4).
%   - RelativeDifferenceTolerance: relative difference tolerance for
%   convergence of the Generalized Forward-Backward iterations (default: 10^-4)
%   - L: number of scales of the wavelets (default: 3).
%   - isometric: use isometric wavelet transform (default: 0).
%   - sourcesShape: when dealing with 2D images, shape of the images (ex:
%   [128, 128])
%   - dimensions: dimensions on which to compute the wavelets (up to 2
%   different dimensions, default: all but first dimension)

function res = redWaveThresholding_m(x, lambda, options)


if nargin < 3
   options = []; 
end

if ~isfield(options, 'MaximumIteration')
    options.MaximumIteration = 24;
end

if ~isfield(options, 'RelativeDifferenceTolerance')
    options.RelativeDifferenceTolerance = 0.0001;
end

if ~isfield(options, 'qmf')
    options.qmf = [0.482962913145, 0.836516303738, 0.224143868042, -0.129409522551];
end

if ~isfield(options, 'L')
    options.L = 3;
end

if ~isfield(options, 'isometric')
    options.isometric = 0;
end

if ~isfield(options, 'dimensions')
    options.dimensions = 2 : ndims(x);
end

gradient = @(u) redWave(redWave(u, -1, options) - x, 1, options);


y = redWave(x, 1, options);
prev_y = y;
t = 1;
tau = 1;
y = y - soft(y, tau * lambda);


for k = 1 : options.MaximumIteration - 1
    
    t_next = (1 + sqrt(1 + 4 * t^2)) / 2;
    alpha = (t - 1) / t_next;
    t = t_next;
    z = (1 + alpha) * y - alpha * prev_y;
    prev_y = y;
    z = z - tau * gradient(z);
    y = z - soft(z, tau * lambda);
    
    
end

res = x - redWave(y, -1, options);


end

function y = soft(x, lambda)
    y = sign(x) .* max(0, abs(x) - lambda);
end

